local _, nRep = ...

local function RegisterDefaultSetting(key, value)
    if ( nReputationDB == nil ) then
        nReputationDB = {}
    end
    if ( nReputationDB[key] == nil ) then
        nReputationDB[key] = value
    end
end

nReputationMixin = {}

function nReputationMixin:OnLoad()
    self:RegisterEvent("VARIABLES_LOADED")

    self.optionName = "Toggle"
    RegisterDefaultSetting("Toggle", true)
end

function nReputationMixin:TurnOn()
	self:RegisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE")
end

function nReputationMixin:TurnOff()
	self:UnregisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE")
end

function nReputationMixin:OnClick()
    PlaySound(SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON)
    local checked = self:GetChecked()
    self:SetOption(checked)
    self:ApplySetting(checked)
end

function nReputationMixin:Update()
    local currentValue = nReputationDB[self.optionName]
    self:SetChecked(currentValue)
    self:ApplySetting()
end

function nReputationMixin:SetOption()
    nReputationDB[self.optionName] = self:GetChecked()
end

function nReputationMixin:ApplySetting()
    if ( self:GetChecked() ) then
        self:TurnOn()
    else
        self:TurnOff()
    end
end

function nReputationMixin:OnEvent(event, ...)
	if ( event == "CHAT_MSG_COMBAT_FACTION_CHANGE" ) then
        local text = ...

		local pattern_standing_inc = string.gsub(string.gsub(FACTION_STANDING_INCREASED, "(%%s)", "(.+)"), "(%%d)", "(%%d+)")
		local s1, e1, faction, amount = string.find(text, pattern_standing_inc)

		if ( s1 ~= nil and amount ~= nil ) then
			if ( faction ~= GUILD ) then
				self:SetWatched(faction)
			end
        end
    elseif ( event == "VARIABLES_LOADED" ) then
        self:Update()
        self:UnregisterEvent(event)
	end
end

function nReputationMixin:SetWatched(newFaction)
    if ( running ) then
        return
    end

    running = true

    local i = 1
    local wasCollapsed = {}
    local watchedFaction = GetWatchedFactionInfo()

    while i <= GetNumFactions() do
        local name, _, _, _, _, _, _, _, isHeader, isCollapsed, _, _, _, _, _, _ = GetFactionInfo(i)
        if ( isHeader ) then
            if ( name == FACTION_INACTIVE ) then
                break
            end
            if ( isCollapsed ) then
                ExpandFactionHeader(i)
                wasCollapsed[name] = true
            end
        end
        if ( name == newFaction ) then
            if ( watchedFaction ~= newFaction ) then
                SetWatchedFactionIndex(i)
            end
            break
        end
        i = i + 1
    end

    i = 1
    while i <= GetNumFactions() do
        local name, _, _, _, _, _, _, _, isHeader, isCollapsed, _, _, _, _, _, _ = GetFactionInfo(i)
        if ( isHeader and not isCollapsed and wasCollapsed[name] ) then
            CollapseFactionHeader(i)
            wasCollapsed[name] = nil
        end
        i = i + 1
    end

    running = nil
end
